@@ -1,15 +1,29 @@
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, replace
 from enum import IntFlag, auto
-from typing import TYPE_CHECKING, Any, Protocol, Sequence, runtime_checkable
+from typing import TYPE_CHECKING, Any, Sequence
 
-from jetpytools import MISSING
-from typing_extensions import Self
+from jetpytools import MISSING, copy_signature, fallback
+from typing_extensions import Self, TypeVar
 
-from vskernels import Catrom, ComplexScaler, ComplexScalerLike, LeftShift, Scaler, TopShift
+from vskernels import (
+    Bobber,
+    Catrom,
+    ComplexScaler,
+    ComplexScalerLike,
+    LeftShift,
+    MixedScalerProcess,
+    Point,
+    Scaler,
+    TopShift,
+)
 from vstools import (
     ChromaLocation,
     ConstantFormatVideoNode,
+    FieldBased,
+    FieldBasedLike,
     VideoNodeT,
     VSFunctionAllArgs,
     VSFunctionNoArgs,
@@ -17,7 +31,6 @@ from vstools import (
     core,
     normalize_seq,
     vs,
-    vs_object,
 )
 
 __all__ = [
@@ -29,12 +42,43 @@ __all__ = [
     "Deinterlacer",
     "SangNom",
     "SuperSampler",
-    "SupportsBobDeinterlace",
+    "SuperSamplerProcess",
 ]
 
 
+class DeinterlacerKwargs(dict[str, Any]):
+    """
+    A dict-like wrapper that syncs keys with a `Deinterlacer` instance.
+
+    - If a key matches an attribute of `deinterlacer`, the value is set on
+      the object instead of stored in the dict.
+    - Otherwise, the pair is stored normally.
+
+    `update()` and `setdefault()` are overridden to respect this behavior.
+    """
+
+    deinterlacer: Deinterlacer
+    """Deinterlacer object."""
+
+    def __setitem__(self, key: str, value: Any) -> None:
+        if not hasattr(self.deinterlacer, key):
+            return super().__setitem__(key, value)
+        setattr(self.deinterlacer, key, value)
+
+    @copy_signature(dict.update)
+    def update(self, *args: Any, **kwargs: Any) -> None:
+        for k, v in dict(*args, **kwargs).items():
+            self[k] = v
+
+    @copy_signature(dict.setdefault)
+    def setdefault(self, key: str, default: Any = None) -> Any:
+        if key not in self:
+            self[key] = default
+        return self[key]
+
+
 @dataclass(kw_only=True)
-class Deinterlacer(vs_object, ABC):
+class Deinterlacer(Bobber, ABC):
     """
     Abstract base class for deinterlacing operations.
     """
@@ -45,6 +89,10 @@ class Deinterlacer(vs_object, ABC):
     double_rate: bool = True
     """Whether to double the FPS."""
 
+    def __post_init__(self) -> None:
+        self.kwargs = DeinterlacerKwargs()
+        self.kwargs.deinterlacer = self
+
     @property
     @abstractmethod
     def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
@@ -85,31 +133,51 @@ class Deinterlacer(vs_object, ABC):
         """
         return kwargs
 
-    def deinterlace(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def deinterlace(
+        self,
+        clip: vs.VideoNode,
+        *,
+        tff: FieldBasedLike | bool | None = None,
+        double_rate: bool | None = None,
+        **kwargs: Any,
+    ) -> ConstantFormatVideoNode:
         """
         Apply deinterlacing to the given clip.
 
         Args:
             clip: The input clip.
+            tff: Field order of the clip.
+            double_rate: Whether to double the frame rate (True) or retain the original rate (False).
             **kwargs: Additional arguments passed to the plugin function.
 
         Returns:
             Deinterlaced clip.
         """
-        return self._interpolate(clip, self.tff, self.double_rate, False, **kwargs)
+        return self._interpolate(
+            clip,
+            FieldBased.from_param(fallback(tff, self.tff), self.__class__).is_tff,
+            fallback(double_rate, self.double_rate),
+            False,
+            **kwargs,
+        )
 
-    def bob(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+    def bob(
+        self, clip: vs.VideoNode, *, tff: FieldBasedLike | bool | None = None, **kwargs: Any
+    ) -> ConstantFormatVideoNode:
         """
         Apply bob deinterlacing to the given clip.
 
         Args:
             clip: The input clip.
+            tff: Field order of the clip.
             **kwargs: Additional arguments passed to the plugin function.
 
         Returns:
             Deinterlaced clip.
         """
-        return self._interpolate(clip, self.tff, True, False, **kwargs)
+        return self._interpolate(
+            clip, FieldBased.from_param(fallback(tff, self.tff), self.__class__).is_tff, True, False, **kwargs
+        )
 
     def copy(self, **kwargs: Any) -> Self:
         """
@@ -118,19 +186,6 @@ class Deinterlacer(vs_object, ABC):
         return replace(self, **kwargs)
 
 
-@runtime_checkable
-class SupportsBobDeinterlace(Protocol):
-    """
-    Protocol for classes that support bob deinterlacing.
-    """
-
-    __slots__ = ()
-
-    def bob(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode: ...
-
-    def deinterlace(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode: ...
-
-
 @dataclass(kw_only=True)
 class AntiAliaser(Deinterlacer, ABC):
     """
@@ -205,7 +260,7 @@ class AntiAliaser(Deinterlacer, ABC):
 
 
 @dataclass(kw_only=True)
-class SuperSampler(AntiAliaser, Scaler, ABC):
+class SuperSampler(Scaler, AntiAliaser, ABC):
     """
     Abstract base class for supersampling operations.
     """
@@ -246,7 +301,8 @@ class SuperSampler(AntiAliaser, Scaler, ABC):
         """
         assert check_variable(clip, self.scale)
 
-        dest_dimensions = list(self._wh_norm(clip, width, height))
+        dims = self._wh_norm(clip, width, height)
+        dest_dimensions = list(dims)
         sy, sx = shift
 
         cloc = list(ChromaLocation.from_video(clip).get_offsets(clip))
@@ -266,7 +322,10 @@ class SuperSampler(AntiAliaser, Scaler, ABC):
         for x, dim in enumerate(dest_dimensions):
             is_width = (not x and self.transpose_first) or (not self.transpose_first and x)
 
-            while (clip.width if is_width else clip.height) < dim:
+            if is_width:
+                clip = self.transpose(clip)
+
+            while clip.height < dim:
                 delta = max(nshift[x], key=lambda y: abs(y))
                 tff = False if delta < 0 else True if delta > 0 else self.tff
                 offset = -0.25 if tff else 0.25
@@ -277,20 +336,22 @@ class SuperSampler(AntiAliaser, Scaler, ABC):
                     else:
                         nshift[x][y] = (nshift[x][y] + offset) * 2 - cloc[x] / subsampling[x]
 
-                if is_width:
-                    clip = self.transpose(clip)
-
                 clip = self._interpolate(clip, tff, False, True, **kwargs)
 
-                if is_width:
-                    clip = self.transpose(clip)
+            if is_width:
+                clip = self.transpose(clip)
 
         if not self.transpose_first:
             nshift.reverse()
 
+        self._ss_shifts = nshift
+
         if self.noshift:
             noshift = normalize_seq(self.noshift, clip.format.num_planes)
 
+            if all(noshift) and dims == (clip.width, clip.height):
+                return clip
+
             for ns in nshift:
                 for i in range(len(ns)):
                     ns[i] *= not noshift[i]
@@ -418,7 +479,7 @@ class NNEDI3(SuperSampler):
             "pscrn": self.pscrn,
         } | kwargs
 
-    @Scaler.cached_property
+    @Scaler.cachedproperty
     def kernel_radius(self) -> int:
         match self.nsize:
             case 0 | 4:
@@ -539,7 +600,7 @@ class EEDI2(SuperSampler):
             "pp": self.pp,
         } | kwargs
 
-    @Scaler.cached_property
+    @Scaler.cachedproperty
     def kernel_radius(self) -> int:
         return self.maxd
 
@@ -641,9 +702,6 @@ class EEDI3(SuperSampler):
     by limiting edge-directed interpolation to certain pixels.
     """
 
-    opencl: bool = False
-    """Enables the use of the OpenCL variant for processing."""
-
     def _set_sclip_mclip(self, kwargs: dict[str, Any]) -> dict[str, Any]:
         sclip, mclip = kwargs.pop("sclip", MISSING), kwargs.pop("mclip", MISSING)
 
@@ -657,7 +715,7 @@ class EEDI3(SuperSampler):
 
     @property
     def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]:
-        return core.lazy.eedi3m.EEDI3CL if self.opencl else core.lazy.eedi3m.EEDI3
+        return core.lazy.eedi3m.EEDI3
 
     def _interpolate(
         self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
@@ -710,7 +768,7 @@ class EEDI3(SuperSampler):
         if self.vthresh is None:
             self.vthresh = (None, None, None)
 
-        eedi3_kwargs = {
+        kwargs = {
             "alpha": self.alpha,
             "beta": self.beta,
             "gamma": self.gamma,
@@ -723,14 +781,15 @@ class EEDI3(SuperSampler):
             "vthresh1": self.vthresh[1],
             "vthresh2": self.vthresh[2],
             "sclip": self.sclip,
-        }
+            "mclip": self.mclip,
+        } | kwargs
 
-        if not self.opencl:
-            eedi3_kwargs.update(mclip=self.mclip)
+        if kwargs.get("opt") is None:
+            kwargs["opt"] = 2 if kwargs["mclip"] else 3
 
-        return eedi3_kwargs | kwargs
+        return kwargs
 
-    @Scaler.cached_property
+    @Scaler.cachedproperty
     def kernel_radius(self) -> int:
         return self.mdis
 
@@ -806,3 +865,84 @@ class BWDIF(Deinterlacer):
 
     def __vs_del__(self, core_id: int) -> None:
         self.edeint = None
+
+    _static_kernel_radius = 2
+
+
+if TYPE_CHECKING:
+    # Let's assume the specialized SuperSampler isn't abstract
+    class _ConcreteSuperSampler(SuperSampler):
+        @property
+        def _deinterlacer_function(self) -> VSFunctionAllArgs[vs.VideoNode, ConstantFormatVideoNode]: ...
+        def _interpolate(
+            self, clip: vs.VideoNode, tff: bool, double_rate: bool, dh: bool, **kwargs: Any
+        ) -> ConstantFormatVideoNode: ...
+        def get_deint_args(self, **kwargs: Any) -> dict[str, Any]: ...
+else:
+    _ConcreteSuperSampler = SuperSampler
+
+_SuperSamplerWithNNEDI3DefaultT = TypeVar("_SuperSamplerWithNNEDI3DefaultT", bound=SuperSampler, default=NNEDI3)
+
+
+class SuperSamplerProcess(
+    MixedScalerProcess[_SuperSamplerWithNNEDI3DefaultT, Point], _ConcreteSuperSampler, partial_abstract=True
+):
+    """
+    A utility SuperSampler class that applies a given function to a supersampled clip,
+    then downsamples it back using Point.
+
+    If used without a specified scaler, it defaults to inheriting from `NNEDI3`.
+    """
+
+    _default_scaler = NNEDI3
+
+    def __init__(
+        self,
+        *,
+        function: VSFunctionNoArgs[vs.VideoNode, vs.VideoNode],
+        noshift: bool | Sequence[bool] = True,
+        **kwargs: Any,
+    ) -> None:
+        """
+        Initialize the SuperSamplerProcess.
+
+        Note:
+            Chroma planes will not align properly during processing.
+            Avoid using this class if accurate chroma placement relative to luma is required.
+
+        Example:
+        ```py
+        processed = SuperSamplerProcess[NNEDI3](function=lambda clip: cool_function(clip, ...)).supersample(
+            src, rfactor=2
+        )
+        ```
+
+        Args:
+            function: A function to apply on the supersampled clip.
+            noshift: Disables sub-pixel shifting after supersampling.
+
+                   - `bool`: Applies to both luma and chroma.
+                   - `Sequence[bool]`: First for luma, second for chroma.
+
+            **kwargs: Additional arguments to the specialized SuperSampler.
+        """
+        super().__init__(function=function, noshift=noshift, **kwargs)
+
+    def scale(
+        self,
+        clip: vs.VideoNode,
+        width: int | None = None,
+        height: int | None = None,
+        shift: tuple[TopShift, LeftShift] = (0, 0),
+        **kwargs: Any,
+    ) -> ConstantFormatVideoNode:
+        ss_clip = super().scale(clip, width, height, shift, **kwargs)
+
+        processed = self.function(ss_clip)
+
+        return self._others[0].scale(
+            processed,
+            clip.width,
+            clip.height,
+            tuple([round(s - 1e-6) for s in dim_shifts] for dim_shifts in reversed(self._ss_shifts)),  # type: ignore[return-value, arg-type]
+        )
