@@ -3,7 +3,7 @@ from __future__ import annotations
 from enum import auto
 from typing import Any, Callable, ClassVar, Iterable, Literal, Protocol, Sequence, TypeAlias, TypeVar, overload
 
-from jetpytools import MISSING, CustomEnum, FuncExceptT, MissingT, fallback, inject_self
+from jetpytools import MISSING, CustomEnum, FuncExcept, MissingT, fallback, inject_self
 from typing_extensions import deprecated
 
 from vsexprtools import norm_expr
@@ -14,7 +14,7 @@ from vstools import (
     ColorRange,
     ConstantFormatVideoNode,
     ConvMode,
-    PlanesT,
+    Planes,
     UnsupportedColorFamilyError,
     check_variable,
     core,
@@ -50,6 +50,7 @@ Tuple representing lower and upper edge limits for each plane.
 Format: (low, high)
 
 Each element can be:
+
 - A float: the same limit is applied to all planes.
 - A sequence of floats: individual limits for each plane.
 - True: use the default legal range per plane.
@@ -63,7 +64,7 @@ class _GrainerFunc(Protocol):
     """
 
     def __call__(
-        self, clip: vs.VideoNode, strength: Sequence[float], planes: PlanesT, **kwargs: Any
+        self, clip: vs.VideoNode, strength: Sequence[float], planes: Planes, **kwargs: Any
     ) -> vs.VideoNode: ...
 
 
@@ -442,7 +443,7 @@ def _apply_grainer(
     post_process: Callable[..., vs.VideoNode] | Iterable[Callable[..., vs.VideoNode]] | None,
     protect_neutral_chroma: bool | None,
     luma_scaling: float | None,
-    func: FuncExceptT,
+    func: FuncExcept,
     **kwargs: Any,
 ) -> vs.VideoNode:
     # Normalize params
@@ -529,7 +530,7 @@ def _protect_pixel_range(
     low: list[float],
     high: list[float],
     blend: float = 0.0,
-    func: FuncExceptT | None = None,
+    func: FuncExcept | None = None,
 ) -> vs.VideoNode:
     if not blend:
         expr = "y neutral - abs A! x A@ - {lo} < x A@ + {hi} > or neutral y ?"
@@ -550,14 +551,14 @@ def _protect_neutral_chroma(
     grained: vs.VideoNode,
     base_clip: vs.VideoNode,
     blend: float = 0.0,
-    func: FuncExceptT | None = None,
+    func: FuncExcept | None = None,
 ) -> vs.VideoNode:
     match clip.format.color_family:
         case vs.YUV:
             if not blend:
-                expr = "x neutral = y neutral = and range_max 0 ?"
+                expr = "x neutral = y neutral = and mask_max 0 ?"
             else:
-                expr = "x neutral - abs {blend} / 1 min 1 swap - y neutral - abs {blend} / 1 min 1 swap - * range_max *"
+                expr = "x neutral - abs {blend} / 1 min 1 swap - y neutral - abs {blend} / 1 min 1 swap - * mask_max *"
 
             mask = norm_expr([get_u(clip), get_v(clip)], expr, func=func, blend=blend)
 
@@ -566,7 +567,7 @@ def _protect_neutral_chroma(
             )
         case vs.RGB:
             return core.std.MaskedMerge(
-                grained, base_clip, norm_expr(split(clip), "x y = x z = and range_max *", func=func)
+                grained, base_clip, norm_expr(split(clip), "x y = x z = and mask_max *", func=func)
             )
         case _:
             raise UnsupportedColorFamilyError("Can't use `protect_neutral_chroma=True` when input clip is GRAY", func)
