@@ -1,3 +1,4 @@
+from copy import deepcopy
 from functools import partial
 from math import factorial
 from typing import Any, Iterable, Literal, MutableMapping, Protocol, cast
@@ -5,7 +6,7 @@ from typing import Any, Iterable, Literal, MutableMapping, Protocol, cast
 from jetpytools import CustomIntEnum
 from typing_extensions import Self
 
-from vsaa import NNEDI3, Deinterlacer
+from vsaa import NNEDI3
 from vsdeband import Grainer
 from vsdenoise import (
     DFTTest,
@@ -16,16 +17,17 @@ from vsdenoise import (
     MVToolsPreset,
     mc_clamp,
     prefilter_to_full_range,
+    refine_blksize,
 )
 from vsexprtools import norm_expr
-from vskernels import Catrom
+from vskernels import Bobber, BobberLike, Catrom
 from vsmasktools import Coordinates, Morpho
 from vsrgtools import BlurMatrix, gauss_blur, median_blur, remove_grain, repair, unsharpen
 from vstools import (
     ConstantFormatVideoNode,
     ConvMode,
     FieldBased,
-    FieldBasedT,
+    FieldBasedLike,
     KwargsT,
     UnsupportedFieldBasedError,
     UnsupportedVideoFormatError,
@@ -33,6 +35,7 @@ from vstools import (
     check_variable,
     core,
     fallback,
+    get_y,
     normalize_seq,
     sc_detect,
     scale_delta,
@@ -311,7 +314,7 @@ class QTempGaussMC(vs_object):
         self,
         clip: vs.VideoNode,
         input_type: InputType = InputType.INTERLACE,
-        tff: FieldBasedT | bool | None = None,
+        tff: FieldBasedLike | bool | None = None,
     ) -> None:
         """
         Args:
@@ -380,7 +383,9 @@ class QTempGaussMC(vs_object):
         *,
         force_tr: int = 1,
         preset: MVToolsPreset = MVToolsPreset.HQ_SAD,
+        chroma: bool = True,
         blksize: int | tuple[int, int] = 16,
+        overlap: int | tuple[int, int] = 2,
         refine: int = 1,
         thsad_recalc: int | None = None,
         thscd: int | tuple[int | None, int | float | None] | None = (180, 38.5),
@@ -391,7 +396,9 @@ class QTempGaussMC(vs_object):
         Args:
             force_tr: Always analyze motion to at least this, even if otherwise unnecessary.
             preset: MVTools preset defining base values for the MVTools object.
+            chroma: Whether to consider chroma in motion vector calculations.
             blksize: Size of a block. Larger blocks are less sensitive to noise, are faster, but also less accurate.
+            overlap: The blksize divisor for block overlap. Larger overlapping reduces blocking artifacts.
             refine: Number of times to recalculate motion vectors with halved block size.
             thsad_recalc: Only bad quality new vectors with a SAD above this will be re-estimated by search. thsad value
                 is scaled to 8x8 block size.
@@ -403,7 +410,9 @@ class QTempGaussMC(vs_object):
 
         self.analyze_tr = force_tr
         self.analyze_preset = preset
-        self.analyze_blksize = blksize if isinstance(blksize, tuple) else (blksize, blksize)
+        self.analyze_chroma = chroma
+        self.analyze_blksize = blksize
+        self.analyze_overlap = overlap
         self.analyze_refine = refine
         self.analyze_thsad_recalc = thsad_recalc
         self.analyze_thscd = thscd
@@ -417,6 +426,7 @@ class QTempGaussMC(vs_object):
         func: _DenoiseFuncTr | VSFunctionKwArgs[vs.VideoNode, vs.VideoNode] = partial(DFTTest().denoise, sigma=8),
         mode: NoiseProcessMode = NoiseProcessMode.IDENTIFY,
         deint: NoiseDeintMode = NoiseDeintMode.GENERATE,
+        mc_denoise: bool = True,
         stabilize: tuple[float, float] | Literal[False] = (0.6, 0.2),
         func_comp_args: KwargsT | None = None,
         stabilize_comp_args: KwargsT | None = None,
@@ -429,6 +439,7 @@ class QTempGaussMC(vs_object):
             func: Denoising function to use.
             mode: Noise handling method to use.
             deint: Noise deinterlacing method to use.
+            mc_denoise: Whether to perform motion-compensated denoising.
             stabilize: Weights to use when blending source noise with compensated noise.
             func_comp_args: Arguments passed to [MVTools.compensate][vsdenoise.mvtools.mvtools.MVTools.compensate] for
                 denoising.
@@ -440,6 +451,7 @@ class QTempGaussMC(vs_object):
         self.denoise_func = func
         self.denoise_mode = mode
         self.denoise_deint = deint
+        self.denoise_mc_denoise = mc_denoise
         self.denoise_stabilize: tuple[float, float] | Literal[False] = stabilize
         self.denoise_func_comp_args = fallback(func_comp_args, KwargsT())
         self.denoise_stabilize_comp_args = fallback(stabilize_comp_args, KwargsT())
@@ -451,7 +463,7 @@ class QTempGaussMC(vs_object):
         *,
         tr: int = 2,
         thsad: int | tuple[int, int] = 640,
-        bobber: Deinterlacer = NNEDI3(nsize=1),
+        bobber: BobberLike = NNEDI3(nsize=1),
         noise_restore: float = 0.0,
         degrain_args: KwargsT | None = None,
         mask_args: KwargsT | None | Literal[False] = None,
@@ -473,7 +485,12 @@ class QTempGaussMC(vs_object):
 
         self.basic_tr = tr
         self.basic_thsad = thsad
-        self.basic_bobber = bobber.copy(tff=self.tff, double_rate=self.double_rate)
+
+        self.basic_bobber = (
+            deepcopy(bobber) if isinstance(bobber, Bobber) else Bobber.ensure_obj(bobber, self.__class__)
+        )
+        self.basic_bobber.kwargs.update(tff=self.tff, double_rate=self.double_rate)
+
         self.basic_noise_restore = noise_restore
         self.basic_degrain_args = fallback(degrain_args, KwargsT())
         self.basic_mask_shimmer_args = fallback(mask_shimmer_args, KwargsT())
@@ -485,7 +502,7 @@ class QTempGaussMC(vs_object):
         self,
         *,
         tr: int = 1,
-        bobber: Deinterlacer | None = None,
+        bobber: BobberLike | None = None,
         mode: SourceMatchMode = SourceMatchMode.NONE,
         similarity: float = 0.5,
         enhance: float = 0.5,
@@ -504,7 +521,16 @@ class QTempGaussMC(vs_object):
         """
 
         self.match_tr = tr
-        self.match_bobber = fallback(bobber, self.basic_bobber).copy(tff=self.tff, double_rate=self.double_rate)
+
+        if bobber is None:
+            self.match_bobber = deepcopy(self.basic_bobber)
+        elif isinstance(bobber, Bobber):
+            self.match_bobber = deepcopy(bobber)
+        else:
+            self.match_bobber = Bobber.ensure_obj(bobber, self.__class__)
+
+        self.match_bobber.kwargs.update(tff=self.tff, double_rate=self.double_rate)
+
         self.match_mode = mode
         self.match_similarity = similarity
         self.match_enhance = enhance
@@ -687,42 +713,40 @@ class QTempGaussMC(vs_object):
         if not erosion_distance:
             return flt
 
-        iter1 = 1 + (erosion_distance + 1) // 3
-        iter2 = 1 + (erosion_distance + 2) // 3
-
-        over1 = over_dilation // 3
-        over2 = over_dilation % 3
+        iterations = (1 + (erosion_distance + 1) // 3, 1 + (erosion_distance + 2) // 3)
+        over_dilations = (over_dilation // 3, over_dilation % 3)
 
         diff = src.std.MakeDiff(flt)
 
-        opening = Morpho.minimum(diff, iterations=iter1, coords=Coordinates.VERTICAL)
-        closing = Morpho.maximum(diff, iterations=iter1, coords=Coordinates.VERTICAL)
+        closing = Morpho.maximum(diff, iterations=iterations[0], coords=Coordinates.VERTICAL)
+        opening = Morpho.minimum(diff, iterations=iterations[0], coords=Coordinates.VERTICAL)
 
-        if erosion_distance % 3:
-            opening = Morpho.deflate(opening)
+        if erosion_residual := erosion_distance % 3:
             closing = Morpho.inflate(closing)
+            opening = Morpho.deflate(opening)
 
-            if erosion_distance % 3 == 2:
-                opening = median_blur(opening)
+            if erosion_residual == 2:
                 closing = median_blur(closing)
+                opening = median_blur(opening)
 
-        opening = Morpho.maximum(opening, iterations=iter2, coords=Coordinates.VERTICAL)
-        closing = Morpho.minimum(closing, iterations=iter2, coords=Coordinates.VERTICAL)
+        closing = Morpho.minimum(closing, iterations=iterations[1], coords=Coordinates.VERTICAL)
+        opening = Morpho.maximum(opening, iterations=iterations[1], coords=Coordinates.VERTICAL)
 
         if over_dilation:
-            opening = Morpho.maximum(opening, iterations=over1)
-            closing = Morpho.minimum(closing, iterations=over1)
+            closing = Morpho.minimum(closing, iterations=over_dilations[0])
+            opening = Morpho.maximum(opening, iterations=over_dilations[0])
 
-            opening = Morpho.inflate(opening, iterations=over2)
-            closing = Morpho.deflate(closing, iterations=over2)
+            closing = Morpho.deflate(closing, iterations=over_dilations[1])
+            opening = Morpho.inflate(opening, iterations=over_dilations[1])
 
         return norm_expr(
-            [flt, diff, opening, closing],
-            "y neutral - abs {thr} > y a neutral min z neutral max clip y ? neutral - x +",
+            [flt, diff, closing, opening],
+            "x y neutral - abs {thr} > y z neutral min a neutral max clip y ? neutral - +",
             thr=scale_delta(threshold, 8, flt),
+            func=self._mask_shimmer,
         )
 
-    def _interpolate(self, clip: vs.VideoNode, bobber: Deinterlacer) -> ConstantFormatVideoNode:
+    def _interpolate(self, clip: vs.VideoNode, bobber: Bobber) -> ConstantFormatVideoNode:
         assert check_variable(clip, self._interpolate)
 
         if self.input_type != self.InputType.PROGRESSIVE:
@@ -772,10 +796,13 @@ class QTempGaussMC(vs_object):
     def _apply_prefilter(self) -> None:
         self.draft = Catrom(tff=self.tff).bob(self.clip) if self.input_type == self.InputType.INTERLACE else self.clip
 
+        search = self.draft
+
+        if not self.analyze_chroma:
+            search = get_y(search)
+
         if self.input_type == self.InputType.REPAIR:
-            search = BlurMatrix.BINOMIAL()(self.draft, mode=ConvMode.VERTICAL)
-        else:
-            search = self.draft
+            search = BlurMatrix.BINOMIAL()(search, mode=ConvMode.VERTICAL)
 
         if self.prefilter_tr:
             scenechange = self.prefilter_sc_threshold is not False
@@ -801,6 +828,7 @@ class QTempGaussMC(vs_object):
                     lim1=lim1,
                     lim2=lim2,
                     lim3=lim3,
+                    func=self._apply_prefilter,
                 )
         else:
             blurred = smoothed
@@ -811,41 +839,33 @@ class QTempGaussMC(vs_object):
         self.prefilter_output = blurred
 
     def _apply_analyze(self) -> None:
-        def _floor_div_tuple(x: tuple[int, int]) -> tuple[int, int]:
-            return x[0] // 2, x[1] // 2
-
         tr = max(
-            1,
             self.analyze_tr,
             self.denoise_tr,
             self.basic_tr,
             self.match_tr,
             self.limit_radius
             if self.limit_mode in (self.SharpLimitMode.TEMPORAL_PRESMOOTH, self.SharpLimitMode.TEMPORAL_POSTSMOOTH)
-            else 0,
+            else 1,
             self.final_tr,
         )
 
-        blksize = self.analyze_blksize
+        blksize, overlap = self.analyze_blksize, self.analyze_overlap
         thsad_recalc = fallback(
             self.analyze_thsad_recalc,
             round((self.basic_thsad[0] if isinstance(self.basic_thsad, tuple) else self.basic_thsad) / 2),
         )
 
-        self.mv = MVTools(self.draft, self.prefilter_output, **self.analyze_preset)
-        self.mv.analyze(tr=tr, blksize=blksize, overlap=_floor_div_tuple(blksize))
+        self.mv = MVTools(self.draft, self.prefilter_output, chroma=self.analyze_chroma, **self.analyze_preset)
+        self.mv.analyze(tr=tr, blksize=blksize, overlap=refine_blksize(blksize, overlap))
 
         for _ in range(self.analyze_refine):
-            blksize = _floor_div_tuple(blksize)
-            overlap = _floor_div_tuple(blksize)
-
-            self.mv.recalculate(thsad=thsad_recalc, blksize=blksize, overlap=overlap)
+            blksize = refine_blksize(blksize)
+            self.mv.recalculate(thsad=thsad_recalc, blksize=blksize, overlap=refine_blksize(blksize, overlap))
 
     def _apply_denoise(self) -> None:
-        if not self.denoise_mode:
-            self.denoise_output = self.clip
-        else:
-            if self.denoise_tr:
+        if self.denoise_mode:
+            if self.denoise_mc_denoise:
                 denoised = self.mv.compensate(
                     tr=self.denoise_tr,
                     thscd=self.analyze_thscd,
@@ -853,7 +873,9 @@ class QTempGaussMC(vs_object):
                     **self.denoise_func_comp_args,
                 )
             else:
-                denoised = cast(ConstantFormatVideoNode, self.denoise_func(self.draft))
+                denoised = self.denoise_func(self.draft, tr=self.denoise_tr)
+
+            denoised = cast(ConstantFormatVideoNode, denoised)
 
             if self.input_type == self.InputType.INTERLACE:
                 denoised = reinterlace(denoised, self.tff)
@@ -876,9 +898,13 @@ class QTempGaussMC(vs_object):
                             noise_new = Grainer.GAUSS(
                                 noise_source, 2048, protect_edges=False, protect_neutral_chroma=False, neutral_out=True
                             )
-                            noise_new = norm_expr([noise_max, noise_min, noise_new], "x y - z * range_size / y +")
+                            noise_new = norm_expr(
+                                [noise_max, noise_min, noise_new],
+                                "x y - z * range_size / y +",
+                                func=self._apply_denoise,
+                            )
 
-                            noise = core.std.Interleave([noise_source, noise_new]).std.DoubleWeave(self.tff)[::2]
+                            noise = reweave(noise_source, noise_new, self.tff)
 
                 if self.denoise_stabilize:
                     weight1, weight2 = self.denoise_stabilize
@@ -897,10 +923,11 @@ class QTempGaussMC(vs_object):
                         "x neutral - abs y neutral - abs > x y ? {weight1} * x y + {weight2} * +",
                         weight1=weight1,
                         weight2=weight2,
+                        func=self._apply_denoise,
                     )
 
             self.noise = noise
-            self.denoise_output = denoised if self.denoise_mode == self.NoiseProcessMode.DENOISE else self.clip
+        self.denoise_output = denoised if self.denoise_mode == self.NoiseProcessMode.DENOISE else self.clip
 
     def _apply_basic(self) -> None:
         self.bobbed = self._interpolate(self.denoise_output, self.basic_bobber)
@@ -949,7 +976,7 @@ class QTempGaussMC(vs_object):
             binomial_coeff = factorial(tr_f) // factorial(tr) // factorial(tr_f - tr)
             error_adj = 2**tr_f / (binomial_coeff + self.match_similarity * (2**tr_f - binomial_coeff))
 
-            return norm_expr([clip, ref], "y {adj} 1 + * x {adj} * -", adj=error_adj)
+            return norm_expr([clip, ref], "y 1 {adj} + * x {adj} * -", adj=error_adj, func=_error_adjustment)
 
         if self.input_type == self.InputType.INTERLACE:
             clip = reinterlace(clip, self.tff)
@@ -1000,6 +1027,7 @@ class QTempGaussMC(vs_object):
         processed_diff = norm_expr(
             [fields_diff, median_blur(fields_diff, mode=ConvMode.VERTICAL)],
             "x neutral - X! y neutral - Y! X@ Y@ xor neutral X@ abs Y@ abs < x y ? ?",
+            func=self._apply_lossless,
         )
         processed_diff = repair.Mode.MINMAX_SQUARE1(processed_diff, remove_grain.Mode.MINMAX_AROUND2(processed_diff))
 
@@ -1023,21 +1051,26 @@ class QTempGaussMC(vs_object):
 
                 clamp = norm_expr(
                     [clip, source_min, source_max],
-                    "y z + 2 / AVG! AVG@ x {undershoot} - x {overshoot} + clip",
+                    "y z + 2 / x {undershoot} - x {overshoot} + clip",
                     undershoot=scale_delta(undershoot, 8, clip),
                     overshoot=scale_delta(overshoot, 8, clip),
+                    func=self._apply_sharpen,
                 )
                 resharp = unsharpen(clip, self.sharp_strength, BlurMatrix.BINOMIAL()(clamp))
 
         if self.sharp_thin:
             median_diff = norm_expr(
-                [clip, median_blur(clip, mode=ConvMode.VERTICAL)], "y x - {thin} * neutral +", thin=self.sharp_thin
+                [clip, median_blur(clip, mode=ConvMode.VERTICAL)],
+                "y x - {thin} * neutral +",
+                thin=self.sharp_thin,
+                func=self._apply_sharpen,
             )
             blurred_diff = BlurMatrix.BINOMIAL(mode=ConvMode.HORIZONTAL)(median_diff)
 
             resharp = norm_expr(
                 [resharp, blurred_diff, BlurMatrix.BINOMIAL()(blurred_diff)],
-                "y neutral - Y! Y@ abs z neutral - abs < x Y@ + x ?",
+                "y neutral - dup abs z neutral - abs < swap x + x ?",
+                func=self._apply_sharpen,
             )
 
         return resharp
@@ -1053,22 +1086,21 @@ class QTempGaussMC(vs_object):
     def _apply_sharpen_limit(self, clip: vs.VideoNode) -> ConstantFormatVideoNode:
         assert check_variable(clip, self._apply_sharpen_limit)
 
-        if self.sharp_mode or self.sharp_thin:
+        if (self.sharp_mode or self.sharp_thin) and self.limit_radius:
             if self.limit_mode in (self.SharpLimitMode.SPATIAL_PRESMOOTH, self.SharpLimitMode.SPATIAL_POSTSMOOTH):
                 if self.limit_radius == 1:
                     clip = repair.Mode.MINMAX_SQUARE1(clip, self.bobbed)
-                elif self.limit_radius > 1:
-                    clip = repair.Mode.MINMAX_SQUARE1(clip, repair.Mode.MINMAX_SQUARE_REF2(clip, self.bobbed))
+                else:
+                    inpand = Morpho.minimum(self.bobbed, iterations=self.limit_radius)
+                    expand = Morpho.maximum(self.bobbed, iterations=self.limit_radius)
+                    clip = norm_expr([clip, inpand, expand], "x y z clip", func=self._apply_sharpen_limit)
 
-            if self.limit_radius and self.limit_mode in (
-                self.SharpLimitMode.TEMPORAL_PRESMOOTH,
-                self.SharpLimitMode.TEMPORAL_POSTSMOOTH,
-            ):
+            if self.limit_mode in (self.SharpLimitMode.TEMPORAL_PRESMOOTH, self.SharpLimitMode.TEMPORAL_POSTSMOOTH):
                 clip = mc_clamp(
                     clip,
                     self.bobbed,
                     self.mv,
-                    clamp=self.limit_clamp,
+                    self.limit_clamp,
                     tr=self.limit_radius,
                     thscd=self.analyze_thscd,
                     **self.limit_comp_args,
@@ -1080,7 +1112,9 @@ class QTempGaussMC(vs_object):
         assert check_variable(clip, self._apply_noise_restore)
 
         if restore and hasattr(self, "noise"):
-            clip = norm_expr([clip, self.noise], "y neutral - {restore} * x +", restore=restore)
+            clip = norm_expr(
+                [clip, self.noise], "x y neutral - {restore} * +", restore=restore, func=self._apply_noise_restore
+            )
 
         return clip
 
@@ -1151,11 +1185,9 @@ class QTempGaussMC(vs_object):
         return self.motion_blur_output
 
     def __vs_del__(self, core_id: int) -> None:
-        for k, v in self.__dict__.items():
-            if isinstance(v, MutableMapping):
-                for k2, v2 in v.items():
-                    if isinstance(v2, vs.VideoNode):
-                        v[k2] = None
-
+        for k, v in self.__dict__.copy().items():
             if isinstance(v, vs.VideoNode):
-                setattr(self, k, None)
+                delattr(self, k)
+
+            if isinstance(v, MutableMapping):
+                v.clear()
