@@ -1,18 +1,18 @@
 from __future__ import annotations
 
-from functools import cache, cached_property
+from functools import cache
 from inspect import signature
 from types import MappingProxyType
-from typing import TYPE_CHECKING, Any, Callable, Generic, Sequence, cast
+from typing import TYPE_CHECKING, Any, Callable, Generic, Sequence
 
 from jetpytools import (
     CustomEnum,
     CustomRuntimeError,
     CustomStrEnum,
     CustomValueError,
-    KwargsT,
     P,
     R,
+    cachedproperty,
     fallback,
     interleave_arr,
     normalize_seq,
@@ -23,7 +23,7 @@ from vskernels import Point
 from vstools import (
     ConstantFormatVideoNode,
     FunctionUtil,
-    PlanesT,
+    Planes,
     UnsupportedVideoFormatError,
     check_progressive,
     check_ref_clip,
@@ -47,7 +47,7 @@ def wnnm(
     ref: vs.VideoNode | None = None,
     merge_factor: float = 0.1,
     self_refine: bool = False,
-    planes: PlanesT = None,
+    planes: Planes = None,
     **kwargs: Any,
 ) -> vs.VideoNode:
     """
@@ -91,13 +91,13 @@ def wnnm(
         ref = depth(ref, 32)
         ref = get_y(ref) if func.luma_only else ref
 
-    denoised = cast(ConstantFormatVideoNode, None)
-    dkwargs = KwargsT(radius=tr, rclip=ref) | kwargs
+    dkwargs = dict[str, Any](radius=tr, rclip=ref) | kwargs
 
-    for i in range(refine + 1):
+    previous = func.work_clip
+    denoised = core.wnnm.WNNM(func.work_clip, sigma, **dkwargs)
+
+    for i in range(refine):
         if i == 0:
-            previous = func.work_clip
-        elif i == 1:
             previous = denoised
         else:
             previous = norm_expr(
@@ -108,7 +108,7 @@ def wnnm(
                 func=func.func,
             )
 
-        if self_refine and denoised:
+        if self_refine:
             dkwargs.update(rclip=denoised)
 
         denoised = core.wnnm.WNNM(previous, sigma, **dkwargs)
@@ -275,7 +275,7 @@ class BM3D(Generic[P, R]):
         A profile tailored for handling very noisy content.
         """
 
-        @cached_property
+        @cachedproperty
         def config(self) -> MappingProxyType[str, MappingProxyType[str, MappingProxyType[str, Any]]]:
             """
             Retrieves the configuration for each BM3D profile.
@@ -530,7 +530,7 @@ def bm3d(
     backend: BM3D.Backend = BM3D.Backend.AUTO,
     basic_args: dict[str, Any] | None = None,
     final_args: dict[str, Any] | None = None,
-    planes: PlanesT = None,
+    planes: Planes = None,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
     """
@@ -599,8 +599,8 @@ def bm3d(
     nsigma = normalize_param_planes(clip, sigma, planes, 0, func)
 
     backend = backend.resolve()
-    nbasic_args = fallback(basic_args, KwargsT())
-    nfinal_args = fallback(final_args, KwargsT())
+    nbasic_args = fallback(basic_args, {})
+    nfinal_args = fallback(final_args, {})
 
     matrix_rgb2opp = kwargs.pop("matrix_rgb2opp", BM3D.matrix_rgb2opp)
     matrix_opp2rgb = kwargs.pop("matrix_rgb2opp", BM3D.matrix_opp2rgb)
