@@ -6,8 +6,8 @@ from jetpytools import CustomNotImplementedError, CustomRuntimeError, CustomStrE
 
 from vsaa import BWDIF, NNEDI3, Deinterlacer
 from vsexprtools import norm_expr
-from vskernels import Point
-from vsmasktools import FDoG, GenericMaskT, Morpho, adg_mask, normalize_mask, strength_zones_mask
+from vskernels import Box, Point
+from vsmasktools import FDoG, MaskLike, Morpho, adg_mask, normalize_mask, strength_zones_mask
 from vsrgtools import MeanMode, gauss_blur, repair
 from vsscale import DPIR
 from vstools import (
@@ -16,7 +16,7 @@ from vstools import (
     FrameRangeN,
     FrameRangesN,
     InvalidColorFamilyError,
-    PlanesT,
+    Planes,
     check_progressive,
     check_variable,
     check_variable_format,
@@ -54,7 +54,7 @@ class dpir(CustomStrEnum):  # noqa: N801
         clip: vs.VideoNode,
         strength: _StrengthT | Sequence[_StrengthT] = 10,
         zones: Sequence[tuple[FrameRangeN | FrameRangesN, _StrengthT]] | None = None,
-        planes: PlanesT = None,
+        planes: Planes = None,
         **kwargs: Any,
     ) -> ConstantFormatVideoNode:
         """
@@ -96,7 +96,7 @@ class dpir(CustomStrEnum):  # noqa: N801
                     {
                         None: clip,
                         tuple(planes): self.__call__(clip, strength[1], zones, planes, **kwargs),
-                        0: plane_0 if 0 in planes else self.__call__(plane_0, strength[0], zones, **kwargs),
+                        0: self.__call__(plane_0, strength[0], zones, planes, **kwargs),
                     }
                 )
 
@@ -104,13 +104,13 @@ class dpir(CustomStrEnum):  # noqa: N801
                 strength = normalize_param_planes(clip, strength, planes, 0, func)
 
                 return join(
-                    (self.__call__(plane(clip, i), s, zones, **kwargs) for i, s in enumerate(strength)),
+                    [self.__call__(plane(clip, i), s, zones, **kwargs) for i, s in enumerate(strength)],
                     clip.format.color_family,
                 )
 
             raise CustomRuntimeError
 
-        if not strength:
+        if not strength or not planes:
             return clip
 
         if not isinstance(strength, vs.VideoNode):
@@ -139,7 +139,7 @@ class dpir(CustomStrEnum):  # noqa: N801
             out = dpired
 
         if planes != normalize_planes(clip, None):
-            out = join({None: clip, tuple(planes): dpired}, family=clip.format.color_family)
+            out = join({None: clip, tuple(planes): dpired}, clip.format.color_family)
 
         return out
 
@@ -150,7 +150,7 @@ def dpir_mask(
     high: float = 10,
     lines: float | None = None,
     luma_scaling: float = 12,
-    linemask: GenericMaskT | bool = True,
+    linemask: MaskLike | bool = True,
     relative: bool = False,
 ) -> vs.VideoNode:
     y = depth(get_y(clip), 32)
@@ -182,7 +182,7 @@ def deblock_qed(
     alpha: tuple[int | None, int | None] = (1, 1),
     beta: tuple[int | None, int | None] = (2, 2),
     chroma_mode: int = 0,
-    planes: PlanesT = None,
+    planes: Planes = None,
 ) -> ConstantFormatVideoNode:
     """
     A post-processed deblock. Uses full frequencies of Deblock's changes on block borders, but DCT-lowpassed changes on
@@ -205,7 +205,7 @@ def deblock_qed(
     Returns:
         Deblocked clip
     """
-    from vsdeinterlace.utils import reinterlace
+    from vsdeinterlace import weave
 
     assert check_variable(clip, deblock_qed)
 
@@ -225,11 +225,12 @@ def deblock_qed(
         "X 8 % 7 % Y 8 % 7 % and 0 255 ?",
         planes_pp,
         clip.format.replace(sample_type=vs.SampleType.INTEGER, bits_per_sample=8),  # type: ignore
+        func=deblock_qed,
     )
 
-    strong_diff = norm_expr([clip, strong, mask], "z x y - 1.01 * neutral + neutral ?", planes_pp)
+    strong_diff = norm_expr([clip, strong, mask], "z x y - 1.01 * neutral + neutral ?", planes_pp, func=deblock_qed)
     strong_pp = strong_diff.dctf.DCTFilter([1, 1, 0, 0, 0, 0, 0, 0], planes_pp)
-    deblocked = norm_expr([clip, normal, strong_pp, mask], "a y x z neutral - - ?", planes_pp)
+    deblocked = norm_expr([clip, normal, strong_pp, mask], "a y x z neutral - - ?", planes_pp, func=deblock_qed)
 
     if clip.format.color_family is not vs.GRAY:  # type: ignore
         if chroma_mode == 1:
@@ -238,7 +239,7 @@ def deblock_qed(
             deblocked = join(deblocked, strong)
 
     if fieldbased.is_inter:
-        deblocked = reinterlace(deblocked, fieldbased)
+        deblocked = weave(Box().scale(deblocked, height=clip.height // 2), fieldbased)
 
     return deblocked
 
@@ -268,17 +269,18 @@ def mpeg2stinx(
     Returns:
         Clip with cross-field noise reduced.
     """
+    from vsdeinterlace import weave
 
     def _crossfield_repair(clip: vs.VideoNode, bobbed: vs.VideoNode) -> vs.VideoNode:
-        clip = core.std.Interleave([clip] * 2)
+        clip = core.std.Interleave([clip, clip])
 
-        if sw == 1 and sh == 1:
+        if sw == sh == 1:
             repaired = repair(clip, bobbed, 1)
         else:
             inpand, expand = Morpho.inpand(bobbed, sw, sh), Morpho.expand(bobbed, sw, sh)
-            repaired = MeanMode.MEDIAN([clip, inpand, expand])
+            repaired = norm_expr([clip, inpand, expand], "x y z clip", func=mpeg2stinx)
 
-        return repaired.std.SeparateFields(tff).std.SelectEvery(4, (2, 1)).std.DoubleWeave(tff)[::2]
+        return weave(repaired.std.SeparateFields(tff).std.SelectEvery(4, (2, 1)), tff)
 
     def _temporal_limit(src: vs.VideoNode, flt: vs.VideoNode, adj: vs.VideoNode | None) -> vs.VideoNode:
         if limit is None:
@@ -286,9 +288,9 @@ def mpeg2stinx(
 
         assert adj
 
-        diff = norm_expr([core.std.Interleave([src] * 2), adj], "x y - abs", func=mpeg2stinx).std.SeparateFields(tff)
+        diff = norm_expr([core.std.Interleave([src, src]), adj], "x y - abs", func=mpeg2stinx).std.SeparateFields(tff)
         diff = MeanMode.MINIMUM([diff.std.SelectEvery(4, (0, 1)), diff.std.SelectEvery(4, (2, 3))])
-        diff = Morpho.expand(diff, sw=2, sh=1).std.DoubleWeave(tff)[::2]
+        diff = weave(Morpho.expand(diff, sw=2, sh=1), tff)
 
         return norm_expr([flt, src, diff], "z {limit} * LIM! x y LIM@ - y LIM@ + clip", limit=limit, func=mpeg2stinx)
 
