@@ -6,23 +6,20 @@ from __future__ import annotations
 
 from typing import Any, Literal, Sequence, overload
 
-from jetpytools import MISSING, CustomRuntimeError, FuncExceptT, MissingT
+from jetpytools import MISSING, CustomRuntimeError, FuncExcept, MissingT
 
-from vsexprtools import norm_expr
+from vsexprtools import ExprOp, ExprVars, combine_expr, norm_expr
 from vskernels import Catrom, Kernel, KernelLike, Scaler, ScalerLike
 from vsscale import ArtCNN
 from vstools import (
     ConstantFormatVideoNode,
     KwargsNotNone,
-    PlanesT,
+    Planes,
     VSFunctionNoArgs,
     check_ref_clip,
-    check_variable,
     check_variable_format,
     fallback,
     get_color_family,
-    get_subsampling,
-    get_video_format,
     join,
     normalize_planes,
     normalize_seq,
@@ -30,7 +27,7 @@ from vstools import (
     vs,
 )
 
-from .mvtools import MotionVectors, MVTools, MVToolsPreset
+from .mvtools import MotionVectors, MVTools, MVToolsPreset, refine_blksize
 from .prefilters import PrefilterLike
 
 __all__ = [
@@ -49,14 +46,15 @@ def mc_degrain(
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
+    overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
     thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
-    export_globals: Literal[False] = ...,
-    planes: PlanesT = None,
+    export_globals: Literal[False] = False,
+    planes: Planes = None,
 ) -> vs.VideoNode: ...
 
 
@@ -69,14 +67,16 @@ def mc_degrain(
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
+    overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
     thsad2: int | tuple[int | None, int | None] | None = None,
     thsad_recalc: int | None = None,
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
-    export_globals: Literal[True] = ...,
-    planes: PlanesT = None,
+    *,
+    export_globals: Literal[True],
+    planes: Planes = None,
 ) -> tuple[vs.VideoNode, MVTools]: ...
 
 
@@ -89,6 +89,7 @@ def mc_degrain(
     preset: MVToolsPreset = ...,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
+    overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
     thsad2: int | tuple[int | None, int | None] | None = None,
@@ -96,7 +97,7 @@ def mc_degrain(
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
     export_globals: bool = ...,
-    planes: PlanesT = None,
+    planes: Planes = None,
 ) -> vs.VideoNode | tuple[vs.VideoNode, MVTools]: ...
 
 
@@ -108,6 +109,7 @@ def mc_degrain(
     preset: MVToolsPreset = MVToolsPreset.HQ_SAD,
     tr: int = 1,
     blksize: int | tuple[int, int] = 16,
+    overlap: int | tuple[int, int] = 2,
     refine: int = 1,
     thsad: int | tuple[int, int] = 400,
     thsad2: int | tuple[int | None, int | None] | None = None,
@@ -115,7 +117,7 @@ def mc_degrain(
     limit: int | tuple[int | None, int | None] | None = None,
     thscd: int | tuple[int | None, int | None] | None = None,
     export_globals: bool = False,
-    planes: PlanesT = None,
+    planes: Planes = None,
 ) -> vs.VideoNode | tuple[vs.VideoNode, MVTools]:
     """
     Perform temporal denoising using motion compensation.
@@ -131,6 +133,7 @@ def mc_degrain(
         preset: MVTools preset defining base values for the MVTools object. Default is HQ_SAD.
         tr: The temporal radius. This determines how many frames are analyzed before/after the current frame.
         blksize: Size of a block. Larger blocks are less sensitive to noise, are faster, but also less accurate.
+        overlap: The blksize divisor for block overlap. Larger overlapping reduces blocking artifacts.
         refine: Number of times to recalculate motion vectors with halved block size.
         thsad: Defines the soft threshold of block sum absolute differences. Blocks with SAD above this threshold have
             zero weight for averaging (denoising). Blocks with low SAD have highest weight. The remaining weight is
@@ -153,28 +156,21 @@ def mc_degrain(
         Motion compensated and temporally filtered clip with reduced noise. If export_globals is true: A tuple
         containing the processed clip and the MVTools object.
     """
-
-    def _floor_div_tuple(x: tuple[int, int]) -> tuple[int, int]:
-        return x[0] // 2, x[1] // 2
-
     mv_args = preset | KwargsNotNone(search_clip=prefilter)
 
-    blksize = blksize if isinstance(blksize, tuple) else (blksize, blksize)
     thsad_recalc = fallback(thsad_recalc, round((thsad[0] if isinstance(thsad, tuple) else thsad) / 2))
 
-    mv = MVTools(clip, vectors=vectors, planes=planes, **mv_args)
+    mv = MVTools(clip, vectors=vectors, **mv_args)
     mfilter = mfilter(mv.clip) if callable(mfilter) else fallback(mfilter, mv.clip)
 
     if not vectors:
-        mv.analyze(tr=tr, blksize=blksize, overlap=_floor_div_tuple(blksize))
+        mv.analyze(tr=tr, blksize=blksize, overlap=refine_blksize(blksize, overlap))
 
         for _ in range(refine):
-            blksize = _floor_div_tuple(blksize)
-            overlap = _floor_div_tuple(blksize)
+            blksize = refine_blksize(blksize)
+            mv.recalculate(thsad=thsad_recalc, blksize=blksize, overlap=refine_blksize(blksize, overlap))
 
-            mv.recalculate(thsad=thsad_recalc, blksize=blksize, overlap=overlap)
-
-    den = mv.degrain(mfilter, mv.clip, None, tr, thsad, thsad2, limit, thscd)
+    den = mv.degrain(mfilter, mv.clip, None, tr, thsad, thsad2, limit, thscd, planes=planes)
 
     return (den, mv) if export_globals else den
 
@@ -183,31 +179,26 @@ def mc_clamp(
     flt: vs.VideoNode,
     src: vs.VideoNode,
     mv_obj: MVTools,
-    ref: vs.VideoNode | None = None,
     clamp: int | float | tuple[int | float, int | float] = 0,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
-    from vsexprtools import norm_expr
-    from vsrgtools import MeanMode
-
-    assert check_variable(flt, mc_clamp)
-    assert check_variable(src, mc_clamp)
     check_ref_clip(src, flt, mc_clamp)
 
-    ref = fallback(ref, src)
-
     undershoot, overshoot = normalize_seq(clamp, 2)
 
-    backward_comp, forward_comp = mv_obj.compensate(ref, interleave=False, **kwargs)
+    backward_comp, forward_comp = mv_obj.compensate(src, interleave=False, **kwargs)
+    comp_clips = [src, *backward_comp, *forward_comp]
 
-    comp_min = MeanMode.MINIMUM([ref, *backward_comp, *forward_comp])
-    comp_max = MeanMode.MAXIMUM([ref, *backward_comp, *forward_comp])
+    evars = ExprVars(1, len(comp_clips) + 1, expr_src=True)
 
     return norm_expr(
-        [flt, comp_min, comp_max],
-        "x y {undershoot} - z {overshoot} + clip",
+        [flt, *comp_clips],
+        "src0 {comp_min} {undershoot} - {comp_max} {overshoot} + clamp",
         undershoot=scale_delta(undershoot, 8, flt),
         overshoot=scale_delta(overshoot, 8, flt),
+        comp_min=combine_expr(evars, ExprOp.MIN).to_str(),
+        comp_max=combine_expr(evars, ExprOp.MAX).to_str(),
+        func=mc_clamp,
     )
 
 
@@ -220,8 +211,8 @@ def ccd(
     pscale: float = 0.0,
     chroma_upscaler: ScalerLike = ArtCNN.R8F64_Chroma,
     chroma_downscaler: KernelLike = Catrom,
-    planes: PlanesT | MissingT = MISSING,
-    func: FuncExceptT | None = None,
+    planes: Planes | MissingT = MISSING,
+    func: FuncExcept | None = None,
 ) -> vs.VideoNode:
     """
     Camcorder Color Denoise is a VirtualDub filter originally made by Sergey Stolyarevsky.
@@ -271,18 +262,17 @@ def ccd(
 
     planes = normalize_planes(clip, planes)
 
-    if get_subsampling(clip) not in ["444", None]:
-        full = Scaler.ensure_obj(chroma_upscaler, func).scale(clip, clip.width, clip.height)
-    else:
+    if (clip.format.subsampling_w, clip.format.subsampling_h) == (0, 0):
         full = clip
+        pscale = 1.0
+    else:
+        full = Scaler.ensure_obj(chroma_upscaler, func).scale(clip, clip.width, clip.height)
 
-    full_format = get_video_format(full)
+        if (full.format.subsampling_w, full.format.subsampling_h) != (0, 0):  # type: ignore[union-attr]
+            raise CustomRuntimeError("`chroma_upscaler` didn't upscale chroma planes.", func, repr(full))
 
-    if (full_format.subsampling_w, full_format.subsampling_h) != (0, 0):
-        raise CustomRuntimeError("`chroma_upscaler` didn't upscale chroma planes.", func, repr(full_format))
-
-    if get_color_family(clip) != vs.RGB:
-        rgb = vs.core.resize.Point(full, format=full_format.replace(color_family=vs.RGB).id)
+    if get_color_family(full) != vs.RGB:
+        rgb = vs.core.resize.Point(full, format=full.format.replace(color_family=vs.RGB).id)  # type: ignore[union-attr]
     else:
         rgb = full
 
