@@ -41,7 +41,7 @@ from vstools import (
 from .abstract import BoundingBox, DeferredMask, GeneralMask
 from .edge import SobelStd
 from .morpho import Morpho
-from .types import GenericMaskT, XxpandMode
+from .types import MaskLike, XxpandMode
 from .utils import max_planes, normalize_mask
 
 __all__ = [
@@ -59,21 +59,16 @@ __all__ = [
 ]
 
 
-class _BaseCMaskCar(vs_object):
-    clips: list[vs.VideoNode]
-
-    def __vs_del__(self, core_id: int) -> None:
-        self.clips.clear()
-
-
 @dataclass
-class CustomMaskFromClipsAndRanges(GeneralMask, _BaseCMaskCar):
+class CustomMaskFromClipsAndRanges(GeneralMask, vs_object):
     """
     Abstract CustomMaskFromClipsAndRanges interface
     """
 
+    clips: list[vs.VideoNode] = field(init=False)
+
     processing: VSFunctionNoArgs[vs.VideoNode, ConstantFormatVideoNode] = field(
-        default=core.lazy.std.Binarize, kw_only=True
+        default=core.lazy.std.BinarizeMask, kw_only=True
     )
     idx: Indexer | Type[Indexer] = field(default=IMWRI, kw_only=True)
 
@@ -113,6 +108,9 @@ class CustomMaskFromClipsAndRanges(GeneralMask, _BaseCMaskCar):
     @abstractmethod
     def frame_ranges(self, clip: vs.VideoNode) -> list[list[tuple[int, int]]]: ...
 
+    def __vs_del__(self, core_id: int) -> None:
+        self.clips.clear()
+
 
 @dataclass
 class CustomMaskFromFolder(CustomMaskFromClipsAndRanges):
@@ -190,7 +188,7 @@ class HardsubMask(DeferredMask):
             )
             partials_dehardsubbed.append(partials_dehardsubbed[-1].std.MaskedMerge(p, dehardsub_masks[-1]))
 
-            masks[-1] = masks[-1].std.MaskedMerge(masks[-1].std.Invert(), masks[-2])
+            masks[-1] = masks[-1].std.MaskedMerge(masks[-1].std.InvertMask(), masks[-2])
 
         return partials_dehardsubbed, dehardsub_masks
 
@@ -226,7 +224,7 @@ class HardsubSignFades(HardsubMask):
 
     highpass: float
     expand: int
-    edgemask: GenericMaskT
+    edgemask: MaskLike
     expand_mode: XxpandMode
 
     def __init__(
@@ -235,7 +233,7 @@ class HardsubSignFades(HardsubMask):
         bound: BoundingBox | None = None,
         highpass: float = 0.0763,
         expand: int = 8,
-        edgemask: GenericMaskT = SobelStd,
+        edgemask: MaskLike = SobelStd,
         expand_mode: XxpandMode = XxpandMode.RECTANGLE,
         *,
         blur: bool = False,
@@ -488,7 +486,7 @@ class HardsubASS(HardsubMask):
     def _mask(self, clip: vs.VideoNode, ref: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         mask = core.sub.TextFile(ref, self.filename, fontdir=self.fontdir, blend=False).std.PropToClip("_Alpha")
 
-        mask = mask.std.Binarize(1)
+        mask = mask.std.BinarizeMask(1)
 
         mask = iterate(mask, core.lazy.std.Maximum, 3)
         mask = iterate(mask, core.lazy.std.Inflate, 3)
