@@ -4,7 +4,7 @@ from functools import partial, reduce
 from math import sqrt
 from typing import TYPE_CHECKING, Any, Callable, Generic, Literal, Sequence, Union, overload
 
-from jetpytools import CustomIntEnum, CustomStrEnum, FuncExceptT, P, R, cround
+from jetpytools import CustomIntEnum, CustomStrEnum, FuncExcept, P, R, cround
 
 from vsexprtools import ExprOp, norm_expr
 from vskernels import Bilinear, Gaussian, Point, Scaler, ScalerLike
@@ -14,10 +14,10 @@ from vstools import (
     ConvMode,
     CustomValueError,
     KwargsT,
-    OneDimConvModeT,
-    PlanesT,
-    SpatialConvModeT,
-    TempConvModeT,
+    OneDimConvMode,
+    Planes,
+    SpatialConvMode,
+    TempConvMode,
     VSFunctionNoArgs,
     check_ref_clip,
     check_variable,
@@ -55,8 +55,8 @@ def box_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
     passes: int = 1,
-    mode: OneDimConvModeT | TempConvModeT = ConvMode.HV,
-    planes: PlanesT = None,
+    mode: OneDimConvMode | TempConvMode = ConvMode.HV,
+    planes: Planes = None,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
     """
@@ -100,7 +100,7 @@ def box_blur(
     return clip.vszip.BoxBlur(*box_args)
 
 
-def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: PlanesT = None) -> ConstantFormatVideoNode:
+def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: Planes = None) -> ConstantFormatVideoNode:
     assert check_variable_format(clip, side_box_blur)
 
     half_kernel = [(1 if i <= 0 else 0) for i in range(-radius, radius + 1)]
@@ -125,15 +125,17 @@ def side_box_blur(clip: vs.VideoNode, radius: int = 1, planes: PlanesT = None) -
         if not i == j == 2
     )
 
-    return reduce(lambda x, y: norm_expr([x, y, clip], "x z - abs y z - abs < x y ?", planes=planes), intermediates)
+    return reduce(
+        lambda x, y: norm_expr([x, y, clip], "x z - abs y z - abs < x y ?", planes, func=side_box_blur), intermediates
+    )
 
 
 def gauss_blur(
     clip: vs.VideoNode,
     sigma: float | Sequence[float] = 0.5,
     taps: int | None = None,
-    mode: OneDimConvModeT | TempConvModeT = ConvMode.HV,
-    planes: PlanesT = None,
+    mode: OneDimConvMode | TempConvMode = ConvMode.HV,
+    planes: Planes = None,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
     """
@@ -208,7 +210,7 @@ def min_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
     mode: tuple[ConvMode, ConvMode] = (ConvMode.HV, ConvMode.SQUARE),
-    planes: PlanesT = None,
+    planes: Planes = None,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
     """
@@ -261,9 +263,9 @@ def sbr(
     mode: ConvMode = ConvMode.HV,
     blur: _SbrBlurT | vs.VideoNode = BlurMatrix.BINOMIAL,
     blur_diff: _SbrBlurT = BlurMatrix.BINOMIAL,
-    planes: PlanesT = None,
+    planes: Planes = None,
     *,
-    func: FuncExceptT | None = None,
+    func: FuncExcept | None = None,
     **kwargs: Any,
 ) -> ConstantFormatVideoNode:
     """
@@ -321,8 +323,8 @@ def sbr(
 def median_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
-    mode: SpatialConvModeT = ConvMode.SQUARE,
-    planes: PlanesT = None,
+    mode: SpatialConvMode = ConvMode.SQUARE,
+    planes: Planes = None,
 ) -> ConstantFormatVideoNode: ...
 
 
@@ -331,7 +333,7 @@ def median_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
     mode: Literal[ConvMode.SQUARE] = ...,
-    planes: PlanesT = None,
+    planes: Planes = None,
     smart: Literal[True] = ...,
     threshold: float | Sequence[float] | None = None,
     scalep: bool = True,
@@ -340,7 +342,7 @@ def median_blur(
 
 @overload
 def median_blur(
-    clip: vs.VideoNode, radius: int = 1, mode: Literal[ConvMode.TEMPORAL] = ..., planes: PlanesT = None
+    clip: vs.VideoNode, radius: int = 1, mode: Literal[ConvMode.TEMPORAL] = ..., planes: Planes = None
 ) -> ConstantFormatVideoNode: ...
 
 
@@ -349,7 +351,7 @@ def median_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
     mode: ConvMode = ConvMode.SQUARE,
-    planes: PlanesT = None,
+    planes: Planes = None,
     smart: bool = False,
     threshold: float | Sequence[float] | None = None,
     scalep: bool = True,
@@ -360,7 +362,7 @@ def median_blur(
     clip: vs.VideoNode,
     radius: int | Sequence[int] = 1,
     mode: ConvMode = ConvMode.SQUARE,
-    planes: PlanesT = None,
+    planes: Planes = None,
     smart: bool = False,
     threshold: float | Sequence[float] | None = None,
     scalep: bool = True,
@@ -417,13 +419,11 @@ def median_blur(
     for r in radius:
         expr_passes = list[str]()
 
-        for mat in ExprOp.matrix("x", r, mode, [(0, 0)]):
-            rb = len(mat) + 1
-            st = rb - 1
-            sp = rb // 2 - 1
-            dp = st - 2
+        for mat in ExprOp.matrix("x", r, mode):
+            n_samples = len(mat)
+            n_op = n_samples // 2
 
-            expr_passes.append(f"{mat} sort{st} swap{sp} min! swap{sp} max! drop{dp} x min@ max@ clip")
+            expr_passes.append(f"{mat} sort{n_samples} drop{n_op} swap{n_op} drop{n_op}")
 
         expr_plane.append(expr_passes)
 
@@ -533,7 +533,7 @@ def flux_smooth(
     clip: vs.VideoNode,
     temporal_threshold: float | Sequence[float] = 7.0,
     spatial_threshold: float | Sequence[float] | None = None,
-    planes: PlanesT = None,
+    planes: Planes = None,
     scalep: bool = True,
 ) -> ConstantFormatVideoNode:
     """
@@ -615,7 +615,7 @@ def guided_filter(
     thr: float | Sequence[float] = 1 / 3,
     mode: GuidedFilter.Mode = GuidedFilter.Mode.GRADIENT,
     use_gauss: bool = False,
-    planes: PlanesT = None,
+    planes: Planes = None,
     down_ratio: int = 0,
     downscaler: ScalerLike = Point,
     upscaler: ScalerLike = Bilinear,
