@@ -5,19 +5,34 @@ This module implements scalers for ONNX models.
 from __future__ import annotations
 
 from abc import ABC
-from logging import warning
-from typing import TYPE_CHECKING, Any, ClassVar, Literal, Protocol, SupportsFloat, TypeAlias, TypeVar, runtime_checkable
+from dataclasses import Field, asdict, fields, replace
+from importlib.util import find_spec
+from logging import DEBUG, debug, getLogger, warning
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
+    Literal,
+    Protocol,
+    SupportsFloat,
+    TypeAlias,
+    TypeVar,
+    get_args,
+    runtime_checkable,
+)
 
-from jetpytools import KwargsT
+from jetpytools import CustomImportError
+from typing_extensions import Self, deprecated
 
 from vsexprtools import norm_expr
 from vskernels import Bilinear, Catrom, Kernel, KernelLike, ScalerLike
 from vstools import (
+    ColorRange,
     ConstantFormatVideoNode,
     CustomValueError,
     DitherType,
     Matrix,
-    MatrixT,
+    MatrixLike,
     ProcessVariableResClip,
     SPath,
     SPathLike,
@@ -38,9 +53,29 @@ from vstools import (
 if TYPE_CHECKING:
     from vsmlrt import backendT as Backend
 
+    BackendLike = Backend | type[Backend] | str
+    """
+    Type alias for anything that can resolve to a Backend from vs-mlrt.
+
+    This includes:
+    - A string identifier.
+    - A class type subclassing `Backend`.
+    - An instance of a `Backend`.
+    """
+else:
+    BackendLike = Any
+
 from .generic import BaseGenericScaler
 
-__all__ = ["DPIR", "ArtCNN", "BaseOnnxScaler", "GenericOnnxScaler", "Waifu2x", "autoselect_backend"]
+__all__ = [
+    "DPIR",
+    "ArtCNN",
+    "BackendLike",
+    "BaseOnnxScaler",
+    "GenericOnnxScaler",
+    "Waifu2x",
+    "autoselect_backend",
+]
 
 
 _IntT = TypeVar("_IntT", bound=int)
@@ -51,15 +86,21 @@ class _SupportsFP16(Protocol):
     fp16: bool
 
 
-def _clean_keywords(kwargs: dict[str, Any], backend: Backend) -> dict[str, Any]:
-    return {k: v for k, v in kwargs.items() if k in backend.__dataclass_fields__}
+def _clean_keywords(kwargs: dict[str, Any], backend: Any) -> dict[str, Any]:
+    valid_fields = _get_backend_fields(backend)
+    return {k: v for k, v in kwargs.items() if k in valid_fields}
+
+
+def _get_backend_fields(backend: Any) -> dict[str, Field[Any]]:
+    return {f.name: f for f in fields(backend)}
 
 
 def autoselect_backend(**kwargs: Any) -> Backend:
     """
     Try to select the best backend for the current system.
-    If the system has an NVIDIA GPU: TRT > CUDA (ORT) > Vulkan > OpenVINO GPU
-    Else: DirectML (D3D12) > MIGraphX > Vulkan > CPU (ORT) > CPU OpenVINO
+
+    If the system has an NVIDIA GPU: TRT > TRT_RTX > DirectML (D3D12) > NCNN (Vulkan) > CUDA (ORT) > OpenVINO GPU.
+    Else: DirectML (D3D12) > MIGraphX > NCNN (Vulkan) > CPU (ORT) > CPU OpenVINO
 
     Args:
         **kwargs: Additional arguments to pass to the backend.
@@ -106,13 +147,20 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
     Abstract generic scaler class for an ONNX model.
     """
 
+    def __new__(cls, *args: Any, **kwargs: Any) -> Self:
+        if find_spec("vsmlrt") is None:
+            raise CustomImportError(cls, "vsmlrt")
+
+        return super().__new__(cls, *args, **kwargs)
+
     def __init__(
         self,
         model: SPathLike | None = None,
-        backend: Backend | None = None,
+        backend: BackendLike | None = None,
         tiles: int | tuple[int, int] | None = None,
         tilesize: int | tuple[int, int] | None = None,
         overlap: int | tuple[int, int] | None = None,
+        multiple: int = 1,
         max_instances: int = 2,
         *,
         kernel: KernelLike = Catrom,
@@ -131,6 +179,7 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
                 model's behavior may vary when they are used.
             tilesize: The size of each tile when splitting the image (if tiles are enabled).
             overlap: The size of overlap between tiles.
+            multiple: Multiple of the tiles.
             max_instances: Maximum instances to spawn when scaling a variable resolution clip.
             kernel: Base kernel to be used for certain scaling/shifting/resampling operations. Defaults to Catrom.
             scaler: Scaler used for scaling operations. Defaults to kernel.
@@ -142,16 +191,31 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
         if model is not None:
             self.model = str(SPath(model).resolve())
 
+        fp16 = self.kwargs.pop("fp16", True)
+        default_args = {"fp16": fp16, "output_format": int(fp16), "use_cuda_graph": True, "use_cublas": True}
+
+        from vsmlrt import backendT
+
         if backend is None:
-            _fp16 = self.kwargs.pop("fp16", True)
-            _default_args = KwargsT(fp16=_fp16, output_format=int(_fp16), use_cuda_graph=True, use_cublas=True)
-            self.backend = autoselect_backend(**_default_args | self.kwargs)
+            self.backend = autoselect_backend(**default_args | self.kwargs)
+        elif isinstance(backend, type):
+            self.backend = backend(**_clean_keywords(default_args | self.kwargs, backend))
+        elif isinstance(backend, str):
+            backends_map = {b.__name__.lower(): b for b in get_args(backendT)}
+
+            try:
+                backend_t = backends_map[backend.lower().strip()]
+            except KeyError:
+                raise CustomValueError("Unknown backend!", self.__class__, backend)
+
+            self.backend = backend_t(**_clean_keywords(default_args | self.kwargs, backend_t))
         else:
-            self.backend = backend
+            self.backend = replace(backend, **_clean_keywords(self.kwargs, backend))
 
         self.tiles = tiles
         self.tilesize = tilesize
         self.overlap = overlap
+        self.multiple = multiple
 
         if self.overlap is None:
             self.overlap_w = self.overlap_h = 8
@@ -162,6 +226,19 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
 
         self.max_instances = max_instances
 
+        if getLogger().level <= DEBUG:
+            debug(f"{self}: Using {self.backend.__class__.__name__} backend")
+
+            valid_fields = _get_backend_fields(self.backend)
+
+            for k, v in asdict(self.backend).items():
+                debug(f"{self}: {k}={v}, default is {valid_fields[k].default}")
+
+            debug(f"{self}: User tiles: {self.tiles}")
+            debug(f"{self}: User tilesize: {self.tilesize}")
+            debug(f"{self}: User overlap: {(self.overlap_w, self.overlap_h)}")
+            debug(f"{self}: User multiple: {self.multiple}")
+
     def scale(
         self,
         clip: vs.VideoNode,
@@ -203,11 +280,17 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
                     ckwargs[k.removeprefix(prefix)] = kwargs.pop(k)
                     break
 
+        debug(f"{self}: Preprocess kwargs: {preprocess_kwargs}")
+        debug(f"{self}: Postprocess kwargs: {postprocess_kwargs}")
+        debug(f"{self}: Inference kwargs: {inference_kwargs}")
+
         wclip = self.preprocess_clip(clip, **preprocess_kwargs)
 
         if 0 not in {clip.width, clip.height}:
             scaled = self.inference(wclip, **inference_kwargs)
         else:
+            debug(f"{self}: Variable resolution clip detected!")
+
             if not isinstance(self.backend, (Backend.TRT, Backend.TRT_RTX)):
                 raise CustomValueError(
                     "Variable resolution clips can only be processed with TRT Backend!", self.__class__, self.backend
@@ -247,7 +330,7 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
             "tilesize": self.tilesize,
             "width": clip.width,
             "height": clip.height,
-            "multiple": 1,
+            "multiple": self.multiple,
             "overlap_w": self.overlap_w,
             "overlap_h": self.overlap_h,
         } | kwargs
@@ -258,19 +341,31 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
         """
         Performs preprocessing on the clip prior to inference.
         """
+        debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+
+        clip = depth(clip, self._pick_precision(16, 32), vs.FLOAT, **kwargs)
+
+        debug(f"{self}.pre: After pp; Clip format is {clip.format!r}")
 
-        clip = depth(clip, self._pick_precision(16, 32), vs.FLOAT)
         return limiter(clip, func=self.__class__)
 
     def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         """
         Handles postprocessing of the model's output after inference.
         """
+        debug(f"{self}.post: Before pp; Clip format is {clip.format!r}")
 
-        return depth(
-            clip, input_clip, dither_type=DitherType.ORDERED if 0 in {clip.width, clip.height} else DitherType.AUTO
+        clip = depth(
+            clip,
+            input_clip,
+            dither_type=DitherType.ORDERED if 0 in {clip.width, clip.height} else DitherType.AUTO,
+            **kwargs,
         )
 
+        debug(f"{self}.post: After pp; Clip format is {clip.format!r}")
+
+        return clip
+
     def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         """
         Runs inference on the given video clip using the configured model and backend.
@@ -278,22 +373,33 @@ class BaseOnnxScaler(BaseGenericScaler, ABC):
 
         from vsmlrt import inference
 
-        tiles, overlaps = self.calc_tilesize(clip)
+        tilesize, overlaps = self.calc_tilesize(clip)
 
-        return inference(clip, self.model, overlaps, tiles, self.backend, **kwargs)
+        debug(f"{self}: Passing clip to inference: {clip.format!r}")
+        debug(f"{self}: Passing model: {self.model}")
+        debug(f"{self}: Passing tiles size: {tilesize}")
+        debug(f"{self}: Passing overlaps: {overlaps}")
+        debug(f"{self}: Passing extra kwargs: {kwargs}")
+
+        return inference(clip, self.model, overlaps, tilesize, self.backend, **kwargs)
 
     def _pick_precision(self, fp16: _IntT, fp32: _IntT) -> _IntT:
         from vsmlrt import Backend
 
-        return (
+        precision = (
             fp16
             if (isinstance(self.backend, _SupportsFP16) and self.backend.fp16)
             and isinstance(
-                self.backend, (Backend.TRT, Backend.ORT_CPU, Backend.ORT_CUDA, Backend.ORT_DML, Backend.ORT_COREML)
+                self.backend,
+                (Backend.TRT, Backend.TRT_RTX, Backend.ORT_CPU, Backend.ORT_CUDA, Backend.ORT_DML, Backend.ORT_COREML),
             )
             else fp32
         )
 
+        debug(f"{self}: Selecting precision: {get_video_format(precision) if precision > 32 else precision!r}")
+
+        return precision
+
 
 class GenericOnnxScaler(BaseOnnxScaler, partial_abstract=True):
     """
@@ -317,7 +423,7 @@ class BaseArtCNN(BaseOnnxScaler):
 
     def __init__(
         self,
-        backend: Backend | None = None,
+        backend: BackendLike | None = None,
         tiles: int | tuple[int, int] | None = None,
         tilesize: int | tuple[int, int] | None = None,
         overlap: int | tuple[int, int] | None = None,
@@ -344,12 +450,15 @@ class BaseArtCNN(BaseOnnxScaler):
             shifter: Kernel used for shifting operations. Defaults to kernel.
             **kwargs: Additional arguments to pass to the backend. See the vsmlrt backend's docstring for more details.
         """
+        from vsmlrt import ArtCNNModel, models_path
+
         super().__init__(
-            None,
+            (SPath(models_path) / "ArtCNN" / f"{ArtCNNModel(self._model).name}.onnx").to_str(),
             backend,
             tiles,
             tilesize,
             overlap,
+            1,
             max_instances,
             kernel=kernel,
             scaler=scaler,
@@ -357,12 +466,6 @@ class BaseArtCNN(BaseOnnxScaler):
             **kwargs,
         )
 
-    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        from vsmlrt import ArtCNN as mlrt_ArtCNN
-        from vsmlrt import ArtCNNModel
-
-        return mlrt_ArtCNN(clip, self.tiles, self.tilesize, self.overlap, ArtCNNModel(self._model), self.backend)
-
 
 class BaseArtCNNLuma(BaseArtCNN):
     def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
@@ -375,15 +478,20 @@ class BaseArtCNNLuma(BaseArtCNN):
         width: int,
         height: int,
         shift: tuple[float, float] = (0, 0),
-        matrix: MatrixT | None = None,
+        matrix: MatrixLike | None = None,
         copy_props: bool = False,
     ) -> ConstantFormatVideoNode:
+        # Changes compared to BaseGenericScaler are:
+        # - extract luma if input clip is luma only is removed  since this is a no op here
+        # - Chroma planes are scaled accordingly with the artcnn'd luma,
+        #   avoiding getting a luma plane when passing a YUV clip.
+
         if (clip.width, clip.height) != (width, height):
             clip = self.scaler.scale(clip, width, height)  # type: ignore[assignment]
 
         if input_clip.format.color_family == vs.YUV:
             scaled_chroma = self.scaler.scale(input_clip, clip.width, clip.height)
-            clip = join(clip, scaled_chroma, vs.YUV)
+            clip = join(clip, scaled_chroma, prop_src=scaled_chroma)
 
         if shift != (0, 0):
             clip = self.shifter.shift(clip, shift)
@@ -400,19 +508,53 @@ class BaseArtCNNLuma(BaseArtCNN):
 class BaseArtCNNChroma(BaseArtCNN):
     def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         assert check_variable_format(clip, self.__class__)
+        assert clip.format.color_family == vs.YUV
 
         if clip.format.subsampling_h != 0 or clip.format.subsampling_w != 0:
             chroma_scaler = Kernel.ensure_obj(kwargs.pop("chroma_scaler", Bilinear))
 
-            clip = chroma_scaler.resample(
-                clip,
-                clip.format.replace(
-                    subsampling_h=0, subsampling_w=0, sample_type=vs.FLOAT, bits_per_sample=self._pick_precision(16, 32)
-                ),
+            format = clip.format.replace(
+                subsampling_h=0,
+                subsampling_w=0,
+                sample_type=vs.FLOAT,
+                bits_per_sample=self._pick_precision(16, 32),
             )
-            return limiter(clip, func=self.__class__)
+            dither_type = DitherType.ORDERED if DitherType.should_dither(clip.format, format) else DitherType.NONE
 
-        return super().preprocess_clip(clip, **kwargs)
+            debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+
+            clip = limiter(
+                chroma_scaler.resample(clip, **dict[str, Any](format=format, dither_type=dither_type) | kwargs),
+                func=self.__class__,
+            )
+
+            debug(f"{self}.pre: Before pp; Clip format is {clip.format!r}")
+
+            return norm_expr(clip, "x 0.5 +", [1, 2], func=self.__class__)
+
+        return norm_expr(super().preprocess_clip(clip, **kwargs), "x 0.5 +", [1, 2], func=self.__class__)
+
+    def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+        clip = norm_expr(clip, "x 0.5 -", [1, 2], func=self.__class__)
+        return super().postprocess_clip(clip, input_clip, **kwargs)
+
+    def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
+        from vsmlrt import flexible_inference
+
+        tilesize, overlaps = self.calc_tilesize(clip)
+
+        debug(f"{self}: Passing clip to inference: {clip.format!r}")
+        debug(f"{self}: Passing model: {self.model}")
+        debug(f"{self}: Passing tiles size: {tilesize}")
+        debug(f"{self}: Passing overlaps: {overlaps}")
+        debug(f"{self}: Passing extra kwargs: {kwargs}")
+
+        u, v = flexible_inference(clip, self.model, overlaps, tilesize, self.backend, **kwargs)
+
+        debug(f"{self}: Inferenced clip: {u.format!r}")
+        debug(f"{self}: Inferenced clip: {v.format!r}")
+
+        return core.std.ShufflePlanes([clip, u, v], [0, 0, 0], vs.YUV, clip)
 
     def _finish_scale(
         self,
@@ -421,7 +563,7 @@ class BaseArtCNNChroma(BaseArtCNN):
         width: int,
         height: int,
         shift: tuple[float, float] = (0, 0),
-        matrix: MatrixT | None = None,
+        matrix: MatrixLike | None = None,
         copy_props: bool = False,
     ) -> ConstantFormatVideoNode:
         if (clip.width, clip.height) != (width, height):
@@ -443,8 +585,8 @@ class ArtCNN(BaseArtCNNLuma):
     """
     Super-Resolution Convolutional Neural Networks optimised for anime.
 
-    A quick reminder that vs-mlrt does not ship these in the base package.\n
-    You will have to grab the extended models pack or get it from the repo itself.\n
+    A quick reminder that vs-mlrt does not ship these in the base package.
+    You will have to grab the extended models pack or get it from the repo itself.
     (And create an "ArtCNN" folder in your models folder yourself)
 
     https://github.com/Artoriuz/ArtCNN/releases/latest
@@ -463,7 +605,8 @@ class ArtCNN(BaseArtCNNLuma):
 
     class C4F32(BaseArtCNNLuma):
         """
-        This has 4 internal convolution layers with 32 filters each.\n
+        This has 4 internal convolution layers with 32 filters each.
+
         If you need an even faster model.
 
         Example usage:
@@ -490,9 +633,14 @@ class ArtCNN(BaseArtCNNLuma):
 
         _model = 1
 
+    @deprecated(
+        "This model is no longer maintained and has been deprecated. Please use R8F64 instead.",
+        category=DeprecationWarning,
+    )
     class C16F64(BaseArtCNNLuma):
         """
-        Very fast and good enough for AA purposes but the onnx variant is officially deprecated.\n
+        Very fast and good enough for AA purposes but the onnx variant is officially deprecated.
+
         This has 16 internal convolution layers with 64 filters each.
 
         ONNX files available at https://github.com/Artoriuz/ArtCNN/tree/388b91797ff2e675fd03065953cc1147d6f972c2/ONNX
@@ -507,6 +655,10 @@ class ArtCNN(BaseArtCNNLuma):
 
         _model = 2
 
+    @deprecated(
+        "This model is no longer maintained and has been deprecated. Please use R8F64 instead.",
+        category=DeprecationWarning,
+    )
     class C16F64_DS(BaseArtCNNLuma):  # noqa: N801
         """
         The same as C16F64 but intended to also sharpen and denoise.
@@ -523,7 +675,8 @@ class ArtCNN(BaseArtCNNLuma):
 
     class C4F32_Chroma(BaseArtCNNChroma):  # noqa: N801
         """
-        The smaller of the two chroma models.\n
+        The smaller of the two chroma models.
+
         These don't double the input clip and rather just try to enhance the chroma using luma information.
 
         Example usage:
@@ -536,9 +689,14 @@ class ArtCNN(BaseArtCNNLuma):
 
         _model = 4
 
+    @deprecated(
+        "This model is no longer maintained and has been deprecated. Please use R8F64 instead.",
+        category=DeprecationWarning,
+    )
     class C16F64_Chroma(BaseArtCNNChroma):  # noqa: N801
         """
-        The bigger of the two chroma models.\n
+        The bigger of the two chroma models.
+
         These don't double the input clip and rather just try to enhance the chroma using luma information.
 
         Example usage:
@@ -553,7 +711,8 @@ class ArtCNN(BaseArtCNNLuma):
 
     class R16F96(BaseArtCNNLuma):
         """
-        The biggest model. Can compete with or outperform Waifu2x Cunet.\n
+        The biggest model. Can compete with or outperform Waifu2x Cunet.
+
         Also quite a bit slower but is less heavy on vram.
 
         Example usage:
@@ -596,7 +755,8 @@ class ArtCNN(BaseArtCNNLuma):
 
     class R8F64_Chroma(BaseArtCNNChroma):  # noqa: N801
         """
-        The new and fancy big chroma model.\n
+        The new and fancy big chroma model.
+
         These don't double the input clip and rather just try to enhance the chroma using luma information.
 
         Example usage:
@@ -611,8 +771,9 @@ class ArtCNN(BaseArtCNNLuma):
 
     class C4F16(BaseArtCNNLuma):
         """
-        This has 4 internal convolution layers with 16 filters each.\n
-        The currently fastest variant. Not really recommended for any filtering.\n
+        This has 4 internal convolution layers with 16 filters each.
+
+        The currently fastest variant. Not really recommended for any filtering.
         Should strictly be used for real-time applications and even then the other non R ones should be fast enough...
 
         Example usage:
@@ -639,10 +800,29 @@ class ArtCNN(BaseArtCNNLuma):
 
         _model = 11
 
+    class R16F96_Chroma(BaseArtCNNChroma):  # noqa: N801
+        """
+        The biggest and fancy chroma model. Shows almost biblical results on the right sources.
+
+        These don't double the input clip and rather just try to enhance the chroma using luma information.
+
+        Example usage:
+        ```py
+        from vsscale import ArtCNN
+
+        chroma_upscaled = ArtCNN.R16F96_Chroma().scale(clip)
+        ```
+        """
+
+        _model = 12
+
 
 class BaseWaifu2x(BaseOnnxScaler):
     scale_w2x: Literal[1, 2, 4]
+    """Upscaling factor."""
+
     noise: Literal[-1, 0, 1, 2, 3]
+    """Noise reduction level"""
 
     _model: ClassVar[int]
     _static_kernel_radius = 2
@@ -651,7 +831,7 @@ class BaseWaifu2x(BaseOnnxScaler):
         self,
         scale: Literal[1, 2, 4] = 2,
         noise: Literal[-1, 0, 1, 2, 3] = -1,
-        backend: Backend | None = None,
+        backend: BackendLike | None = None,
         tiles: int | tuple[int, int] | None = None,
         tilesize: int | tuple[int, int] | None = None,
         overlap: int | tuple[int, int] | None = None,
@@ -688,6 +868,7 @@ class BaseWaifu2x(BaseOnnxScaler):
             tiles,
             tilesize,
             overlap,
+            1,
             max_instances,
             kernel=kernel,
             scaler=scaler,
@@ -714,15 +895,23 @@ class BaseWaifu2x(BaseOnnxScaler):
 
 class BaseWaifu2xRGB(BaseWaifu2x):
     def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        clip = self.kernel.resample(clip, self._pick_precision(vs.RGBH, vs.RGBS), Matrix.RGB)
+        clip = self.kernel.resample(clip, self._pick_precision(vs.RGBH, vs.RGBS), Matrix.RGB, **kwargs)
         return limiter(clip, func=self.__class__)
 
     def postprocess_clip(self, clip: vs.VideoNode, input_clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         assert check_variable_format(clip, self.__class__)
 
-        if clip.format != get_video_format(input_clip):
-            kwargs = {"dither_type": DitherType.ORDERED} | kwargs
-            clip = self.kernel.resample(clip, input_clip, Matrix.from_video(input_clip, func=self.__class__), **kwargs)
+        if get_video_format(clip) != get_video_format(input_clip):
+            kwargs = (
+                dict[str, Any](
+                    format=input_clip,
+                    matrix=Matrix.from_video(input_clip, func=self.__class__),
+                    range=ColorRange.from_video(input_clip, func=self.__class__),
+                    dither_type=DitherType.ORDERED,
+                )
+                | kwargs
+            )
+            clip = self.kernel.resample(clip, **kwargs)
 
         return clip
 
@@ -969,12 +1158,12 @@ _DPIRColorModel: TypeAlias = int
 
 class BaseDPIR(BaseOnnxScaler):
     _model: ClassVar[tuple[_DPIRGrayModel, _DPIRColorModel]]
-    _static_kernel_radius = 2
+    _static_kernel_radius = 8
 
     def __init__(
         self,
         strength: SupportsFloat | vs.VideoNode = 10,
-        backend: Backend | None = None,
+        backend: BackendLike | None = None,
         tiles: int | tuple[int, int] | None = None,
         tilesize: int | tuple[int, int] | None = None,
         overlap: int | tuple[int, int] | None = None,
@@ -1001,8 +1190,9 @@ class BaseDPIR(BaseOnnxScaler):
             shifter: Kernel used for shifting operations. Defaults to kernel.
             **kwargs: Additional arguments to pass to the backend. See the vsmlrt backend's docstring for more details.
         """
+        from vsmlrt import Backend
+
         self.strength = strength
-        self.multiple = 8
 
         super().__init__(
             None,
@@ -1010,6 +1200,7 @@ class BaseDPIR(BaseOnnxScaler):
             tiles,
             tilesize,
             16 if overlap is None else overlap,
+            8,
             -1,
             kernel=kernel,
             scaler=scaler,
@@ -1017,26 +1208,28 @@ class BaseDPIR(BaseOnnxScaler):
             **kwargs,
         )
 
+        if isinstance(self.backend, Backend.TRT) and not self.backend.force_fp16:
+            self.backend.custom_args.extend(["--precisionConstraints=obey", "--layerPrecisions=Conv_123:fp32"])
+
     def scale(
         self,
         clip: vs.VideoNode,
         width: int | None = None,
         height: int | None = None,
         shift: tuple[float, float] = (0, 0),
+        *,
+        copy_props: bool = True,
         **kwargs: Any,
     ) -> ConstantFormatVideoNode:
         assert check_variable_resolution(clip, self.__class__)
 
-        return super().scale(clip, width, height, shift, **kwargs)
-
-    def calc_tilesize(self, clip: vs.VideoNode, **kwargs: Any) -> tuple[tuple[int, int], tuple[int, int]]:
-        return super().calc_tilesize(clip, **{"multiple": self.multiple} | kwargs)
+        return super().scale(clip, width, height, shift, copy_props=copy_props, **kwargs)
 
     def preprocess_clip(self, clip: vs.VideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
         if get_color_family(clip) == vs.GRAY:
             return super().preprocess_clip(clip, **kwargs)
 
-        clip = self.kernel.resample(clip, self._pick_precision(vs.RGBH, vs.RGBS), Matrix.RGB)
+        clip = self.kernel.resample(clip, self._pick_precision(vs.RGBH, vs.RGBS), Matrix.RGB, **kwargs)
 
         return limiter(clip, func=self.__class__)
 
@@ -1044,37 +1237,58 @@ class BaseDPIR(BaseOnnxScaler):
         assert check_variable_format(clip, self.__class__)
 
         if get_video_format(clip) != get_video_format(input_clip):
-            kwargs = {"dither_type": DitherType.ORDERED} | kwargs
-            clip = self.kernel.resample(clip, input_clip, Matrix.from_video(input_clip, func=self.__class__), **kwargs)
+            kwargs = (
+                dict[str, Any](
+                    format=input_clip,
+                    matrix=Matrix.from_video(input_clip, func=self.__class__),
+                    range=ColorRange.from_video(input_clip, func=self.__class__),
+                    dither_type=DitherType.ORDERED,
+                )
+                | kwargs
+            )
+            clip = self.kernel.resample(clip, **kwargs)
 
         return clip
 
     def inference(self, clip: ConstantFormatVideoNode, **kwargs: Any) -> ConstantFormatVideoNode:
-        from vsmlrt import DPIR as mlrt_dpir  # noqa: N811
-        from vsmlrt import DPIRModel
+        from vsmlrt import DPIRModel, inference, models_path
 
-        args = (
-            self.tiles,
-            self.tilesize,
-            self.overlap,
-            DPIRModel(self._model[0] if clip.format.color_family == vs.GRAY else self._model[1]),
-            self.backend,
-        )
+        # Normalizing the strength clip
+        strength_fmt = clip.format.replace(color_family=vs.GRAY)
+
+        if isinstance(self.strength, vs.VideoNode):
+            self.strength = norm_expr(self.strength, "x 255 /", format=strength_fmt, func=self.__class__)
+        else:
+            self.strength = clip.std.BlankClip(format=strength_fmt.id, color=float(self.strength) / 255, keep=True)
+
+        debug(f"{self}: Passing strength clip format: {self.strength.format!r}")
+
+        # Get model name
+        self.model = (
+            SPath(models_path) / "dpir" / f"{DPIRModel(self._model[clip.format.color_family != vs.GRAY]).name}.onnx"
+        ).to_str()
+
+        # Basic inference args
+        tilesize, overlaps = self.calc_tilesize(clip)
+
+        debug(f"{self}: Passing model: {self.model}")
+        debug(f"{self}: Passing tiles size: {tilesize}")
+        debug(f"{self}: Passing overlaps: {overlaps}")
+        debug(f"{self}: Passing extra kwargs: {kwargs}")
+
+        # Padding
         padding = padder.mod_padding(clip, self.multiple, 0)
 
         if not any(padding) or kwargs.pop("no_pad", False):
-            return mlrt_dpir(clip, self.strength, *args)
+            return inference([clip, self.strength], self.model, overlaps, tilesize, self.backend, **kwargs)
 
         clip = padder.MIRROR(clip, *padding)
-        strength = padder.MIRROR(self.strength, *padding) if isinstance(self.strength, vs.VideoNode) else self.strength
+        strength = padder.MIRROR(self.strength, *padding)
 
-        inferenced = mlrt_dpir(clip, strength, *args)
-
-        return inferenced.std.Crop(*padding)
+        return inference([clip, strength], self.model, overlaps, tilesize, self.backend, **kwargs).std.Crop(*padding)
 
     def __vs_del__(self, core_id: int) -> None:
-        if not TYPE_CHECKING:
-            self.strength = None
+        del self.strength
 
 
 class DPIR(BaseDPIR):
