@@ -34,7 +34,7 @@ class AsyncRenderConf:
 
 
 @overload
-def clip_async_render(
+def clip_async_render(  # pyright: ignore[reportOverlappingOverload]
     clip: vs.VideoNode,
     outfile: BinaryIO | SPathLike | None = None,
     progress: str | Callable[[int, int], None] | None = None,
@@ -59,19 +59,6 @@ def clip_async_render(
 ) -> list[T]: ...
 
 
-@overload
-def clip_async_render(
-    clip: vs.VideoNode,
-    outfile: BinaryIO | SPathLike | None = None,
-    progress: str | Callable[[int, int], None] | None = None,
-    callback: Callable[[int, vs.VideoFrame], T] | None = ...,
-    prefetch: int = 0,
-    backlog: int = -1,
-    y4m: bool | None = None,
-    async_requests: int | bool | AsyncRenderConf = False,
-) -> list[T] | None: ...
-
-
 def clip_async_render(
     clip: vs.VideoNode,
     outfile: BinaryIO | SPathLike | None = None,
@@ -333,7 +320,7 @@ def prop_compare_cb(
     prop: str,
     op: str | Callable[[float, float], bool] | None,
     ref: float | bool,
-    return_frame_n: Literal[False] = ...,
+    return_frame_n: Literal[False] = False,
 ) -> tuple[VideoNodeT, Callable[[int, vs.VideoFrame], bool]]: ...
 
 
@@ -343,7 +330,8 @@ def prop_compare_cb(
     prop: str,
     op: str | Callable[[float, float], bool] | None,
     ref: float | bool,
-    return_frame_n: Literal[True] = ...,
+    *,
+    return_frame_n: Literal[True],
 ) -> tuple[VideoNodeT, Callable[[int, vs.VideoFrame], int | SentinelT]]: ...
 
 
@@ -364,7 +352,6 @@ def prop_compare_cb(
     if isinstance(op, str):
         assert op in _operators
 
-    callback: Callable[[int, vs.VideoFrame], SentinelT | int]
     if one_pix:
         clip = (
             vs.core.std.BlankClip(None, 1, 1, vs.GRAY8 if bool_check else vs.GRAYS, length=src.num_frames)
@@ -401,7 +388,7 @@ def prop_compare_cb(
 
 
 @overload
-def find_prop(
+def find_prop(  # pyright: ignore[reportOverlappingOverload]
     src: vs.VideoNode,
     prop: str,
     op: str | Callable[[float, float], bool] | None,
@@ -453,7 +440,7 @@ def find_prop(
         Frame ranges at the specified conditions.
     """
 
-    prop_src, callback = prop_compare_cb(src, prop, op, ref, True)
+    prop_src, callback = prop_compare_cb(src, prop, op, ref, return_frame_n=True)
 
     aconf = AsyncRenderConf(async_requests, (prop_src.width, prop_src.height) == (1, 1), False)
 
